/**************************************************************************************************
* File: Weather click.c
* File Type: C - Source Code File
* Project Name: Weather click
* Company: (c) mikroElektronika, 2015
* Revision History:
*       - initial release (VM);
* Description:
* Weather click carries the BME280 integrated environmental unit from Bosch.
* It's a sensor that detects humidity, pressure and temperature, specifically designed for low current consumption and long term stability.
* Weather click can communicate with the target MCU either through mikroBUS SPI or I2C interfaces.
* The board is designed to use a 3.3V power supply.
* Test configuration:
*    MCU:             PIC32MX795F512L
*                     http://ww1.microchip.com/downloads/en/DeviceDoc/61156G.pdf
*    dev.board:       EasyPIC Fusion
*                     https://shop.mikroe.com/easypic-fusion
*    Oscillator:      HSI-PLL, 80.000MHz
*    EXT:             Weather click:
*                     http://www.mikroe.com/click/weather
*    SW:              mikroC PRO for PIC32
*                     https://www.mikroe.com/mikroc/#pic32
* NOTES:
*     I2C bus is used for communication with the click.
*/

#include "BME280_Driver.h"
#include "BME280_Defs.h"
#include "resources.h"

// TFT module connections
char TFT_DataPort at LATE;
sbit TFT_RST at LATD7_bit;
sbit TFT_BLED at LATD2_bit;
sbit TFT_RS at LATD9_bit;
sbit TFT_CS at LATD10_bit;
sbit TFT_RD at LATD5_bit;
sbit TFT_WR at LATD4_bit;
char TFT_DataPort_Direction at TRISE;
sbit TFT_RST_Direction at TRISD7_bit;
sbit TFT_BLED_Direction at TRISD2_bit;
sbit TFT_RS_Direction at TRISD9_bit;
sbit TFT_CS_Direction at TRISD10_bit;
sbit TFT_RD_Direction at TRISD5_bit;
sbit TFT_WR_Direction at TRISD4_bit;
// End TFT module connections

char tmp;
char text[20];
float tmp_f;

static void MCU_Init() 
{
  I2C2_Init(100000);                                                        // Initialize I2C on pins RA2 and RA3
  delay_ms(200);
}

static void Display_Init()
{
  TFT_Set_Default_Mode();
  TFT_Init_ILI9341_8bit(320, 240);
  TFT_BLED = 1;
  TFT_Fill_Screen(CL_WHITE);
  TFT_Set_Pen(CL_BLACK, 1);
  TFT_Line(20, 220, 300, 220);
  TFT_Line(20,  46, 300,  46);
  TFT_Set_Font(&HandelGothic_BT21x22_Regular, CL_BLACK, FO_HORIZONTAL);
  TFT_Write_Text("Weather click demo", 42, 14);
  TFT_Set_Font(&Verdana12x13_Regular, CL_BLACK, FO_HORIZONTAL);
  TFT_Write_Text("EasyMx PRO v7 for STM32", 19, 223);
  TFT_Write_Text("www.mikroe.com", 200, 223);
}

static void Display_Values() 
{
  TFT_Set_Pen(CL_WHITE, 1);
  TFT_Set_Brush(1, CL_WHITE,0,0,0,0);
  
  tmp_f = BME280_GetTemperature();
  sprintf(text, "%.2f", tmp_f);
  TFT_Rectangle(180,50,320,70);
  TFT_Write_Text(text,200,50);

  tmp_f = BME280_GetPressure();
  sprintf(&text, "%.2f", tmp_f);
  TFT_Rectangle(180,110,320,130);
  TFT_Write_Text(text,200,110);

  tmp_f = BME280_GetHumidity();
  sprintf(&text, "%.2f", tmp_f);
  TFT_Rectangle(180,80,320,100);
  TFT_Write_Text(text,200,80);
}

void BME280_INIT() 
{
  BME280_SetStandbyTime(BME280_STANDBY_TIME_1_MS);                          // Standby time 1ms
  BME280_SetFilterCoefficient(BME280_FILTER_COEFF_16);                      // IIR Filter coefficient 16
  BME280_SetOversamplingPressure(BME280_OVERSAMP_16X);                      // Pressure x16 oversampling
  BME280_SetOversamplingTemperature(BME280_OVERSAMP_2X);                    // Temperature x2 oversampling
  BME280_SetOversamplingHumidity(BME280_OVERSAMP_1X);                       // Humidity x1 oversampling
  BME280_SetOversamplingMode(BME280_NORMAL_MODE);
}

void main() 
{
  MCU_Init();
  Display_Init();
  delay_ms(1000);
  tmp = BME280_GetID();
  if (tmp != BME280_CHIP_ID) {
    TFT_Write_Text("CHIP ERROR",10,10);
    while (1);                                                              //stop program
  }
  else
  {
    TFT_Set_Font(&HandelGothic_BT21x22_Regular, CL_RED, FO_HORIZONTAL);
    TFT_Write_Text("Temperature [C]",10,50);
    TFT_Set_Font(&HandelGothic_BT21x22_Regular, CL_NAVY, FO_HORIZONTAL);
    TFT_Write_Text("Humidity [%RH]",10,80);
    TFT_Set_Font(&HandelGothic_BT21x22_Regular, CL_BLACK, FO_HORIZONTAL);
    TFT_Write_Text("Pressure [hPa]",10,110);                                //hPa = mBar
  }
  
  delay_ms(200);
  BME280_ReadCalibrationParams();                                           //Read calibration parameters
  BME280_SetOversamplingPressure(BME280_OVERSAMP_1X);
  BME280_SetOversamplingTemperature(BME280_OVERSAMP_1X);
  BME280_SetOversamplingHumidity(BME280_OVERSAMP_1X);
  BME280_SetOversamplingMode(BME280_FORCED_MODE);
  while(BME280_IsMeasuring());
  BME280_ReadMeasurements();
  BME280_INIT();  
  while(1)
  {
     while(BME280_IsMeasuring());
     BME280_ReadMeasurements();
     Display_Values();
     BME280_INIT();
     delay_ms(1500);
  }
}